@@ -7,8 +7,8 @@
 #define SOUP_ADDRESS_H
 
 #include <sys/types.h>
-#include <sys/socket.h>
 
+#include <libsoup/soup-portability.h>
 #include <libsoup/soup-types.h>
 
 #define SOUP_TYPE_ADDRESS            (soup_address_get_type ())
@@ -18,12 +18,9 @@
 #define SOUP_IS_ADDRESS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((obj), SOUP_TYPE_ADDRESS))
 #define SOUP_ADDRESS_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SOUP_TYPE_ADDRESS, SoupAddressClass))
 
-typedef struct SoupAddressPrivate SoupAddressPrivate;
-
 struct SoupAddress {
 	GObject parent;
 
-	SoupAddressPrivate *priv;
 };
 
 typedef struct {
@@ -48,8 +45,27 @@ typedef enum {
 } SoupAddressFamily;
 #endif
 
+/**
+ * SOUP_ADDRESS_ANY_PORT:
+ *
+ * This can be passed to any #SoupAddress method that expects a port,
+ * to indicate that you don't care what port is used.
+ **/
 #define SOUP_ADDRESS_ANY_PORT 0
 
+/**
+ * SoupAddressCallback:
+ * @addr: the #SoupAddress that was resolved
+ * @status: %SOUP_STATUS_OK or %SOUP_STATUS_CANT_RESOLVE
+ * @data: the user data that was passed to
+ * soup_address_resolve_async()
+ *
+ * The callback function passed to soup_address_resolve_async().
+ **/
+typedef void   (*SoupAddressCallback)            (SoupAddress         *addr,
+						  guint                status,
+						  gpointer             data);
+
 GType soup_address_get_type (void);
 
 SoupAddress     *soup_address_new                (const char          *name,
@@ -59,18 +75,19 @@ SoupAddress     *soup_address_new_from_sockaddr  (struct sockaddr     *sa,
 SoupAddress     *soup_address_new_any            (SoupAddressFamily    family,
 						  guint                port);
 
-typedef void   (*SoupAddressCallback)            (SoupAddress         *addr,
-						  guint                status,
-						  gpointer             data);
 void             soup_address_resolve_async      (SoupAddress         *addr,
 						  SoupAddressCallback  callback,
 						  gpointer             user_data);
+void             soup_address_resolve_async_full (SoupAddress         *addr,
+						  GMainContext        *async_context,
+						  SoupAddressCallback  callback,
+						  gpointer             user_data);
 guint            soup_address_resolve_sync       (SoupAddress         *addr);
 
 const char      *soup_address_get_name           (SoupAddress         *addr);
-struct sockaddr *soup_address_get_sockaddr       (SoupAddress         *addr,
-						  int                 *len);
 const char      *soup_address_get_physical       (SoupAddress         *addr);
 guint            soup_address_get_port           (SoupAddress         *addr);
+struct sockaddr *soup_address_get_sockaddr       (SoupAddress         *addr,
+						  int                 *len);
 
 #endif /* SOUP_ADDRESS_H */
